@@ -3,12 +3,13 @@ OC.L10N.register(
     {
     "OAuth 2.0" : "OAuth 2.0",
     "OAuth 2.0 clients" : "OAuth 2.0 klientai",
-    "Name" : "Pavadinimas",
-    "Client Identifier" : "Kliento identifikatorius",
+    "OAuth 2.0 allows external services to request access to {instanceName}." : "OAuth 2.0 leidžia išorinėms tarnyboms užklausti prieigą prie {instanceName}.",
     "Add client" : "Pridėti klientą",
-    "OAuth 2.0 allows external services to request access to %s." : "OAuth 2.0 protokolas leidžia trečiųjų šalių programinei įrangai pasiekti šiuos jūsų duomenis: %s.",
-    "Redirection URI" : "Nukreipimo adresas",
+    "Name" : "Pavadinimas",
+    "Redirection URI" : "Peradresavimo URI",
+    "Add" : "Pridėti",
+    "Client Identifier" : "Kliento identifikatorius",
     "Secret" : "Paslaptis",
-    "Add" : "Pridėti"
+    "Delete" : "Ištrinti"
 },
 "nplurals=4; plural=(n % 10 == 1 && (n % 100 > 19 || n % 100 < 11) ? 0 : (n % 10 >= 2 && n % 10 <=9) && (n % 100 > 19 || n % 100 < 11) ? 1 : n % 1 != 0 ? 2: 3);");
