@@ -2,6 +2,7 @@
 /**
  * @copyright Copyright (c) 2016, ownCloud, Inc.
  *
+ * @author Julius Härtl <jus@bitgrid.net>
  * @author Lukas Reschke <lukas@statuscode.ch>
  * @author Morris Jobke <hey@morrisjobke.de>
  * @author Robin Appelman <robin@icewind.nl>
@@ -19,7 +20,7 @@
  * GNU Affero General Public License for more details.
  *
  * You should have received a copy of the GNU Affero General Public License, version 3,
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program. If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -44,7 +45,7 @@ class Jail extends Wrapper {
 	protected $rootPath;
 
 	/**
-	 * @param array $arguments ['storage' => $storage, 'mask' => $root]
+	 * @param array $arguments ['storage' => $storage, 'root' => $root]
 	 *
 	 * $storage: The storage that will be wrapper
 	 * $root: The folder in the wrapped storage that will become the root folder of the wrapped storage
@@ -441,10 +442,6 @@ class Jail extends Wrapper {
 		return $this->getWrapperStorage()->getETag($this->getUnjailedPath($path));
 	}
 
-	/**
-	 * @param string $path
-	 * @return array
-	 */
 	public function getMetaData($path) {
 		return $this->getWrapperStorage()->getMetaData($this->getUnjailedPath($path));
 	}
@@ -538,4 +535,8 @@ class Jail extends Wrapper {
 			return $count;
 		}
 	}
+
+	public function getDirectoryContent($directory): \Traversable {
+		return $this->getWrapperStorage()->getDirectoryContent($this->getUnjailedPath($directory));
+	}
 }
